from moyopy._base import (  # noqa: F401
    Cell,
    CollinearMagneticCell,
    NonCollinearMagneticCell,
    Operations,
)
from moyopy._data import (
    ArithmeticCrystalClass,
    Centering,
    HallSymbolEntry,
    MagneticSpaceGroupType,
    Setting,
    SpaceGroupType,
    magnetic_operations_from_uni_number,
    operations_from_number,
)  # noqa: F401
from moyopy._dataset import (
    MoyoCollinearMagneticDataset,
    MoyoDataset,
    MoyoNonCollinearMagneticDataset,
)  # noqa: F401
from moyopy._identify import MagneticSpaceGroup, PointGroup, SpaceGroup  # noqa: F401

__version__: str

__all__ = [
    # base
    "Cell",
    "CollinearMagneticCell",
    "NonCollinearMagneticCell",
    "Operations",
    # data
    "Setting",
    "Centering",
    "HallSymbolEntry",
    "SpaceGroupType",
    "MagneticSpaceGroupType",
    "ArithmeticCrystalClass",
    "operations_from_number",
    "magnetic_operations_from_uni_number",
    # dataset
    "MoyoDataset",
    "MoyoCollinearMagneticDataset",
    "MoyoNonCollinearMagneticDataset",
    # identify
    "PointGroup",
    "SpaceGroup",
    "MagneticSpaceGroup",
    # lib
    "__version__",
]



from typing import Any

from moyopy._data import Setting

class PointGroup:
    def __init__(
        self, prim_rotations: list[list[int]], *, basis: list[list[float]] | None = None
    ): ...
    @property
    def arithmetic_number(self) -> int: ...
    @property
    def prim_trans_mat(self) -> list[list[int]]: ...
    # Serialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""

class SpaceGroup:
    def __init__(
        self,
        prim_rotations: list[list[int]],
        prim_translations: list[list[float]],
        *,
        basis: list[list[float]] | None = None,
        setting: Setting | None = None,
        epsilon: float = 1e-4,
    ): ...
    @property
    def number(self) -> int: ...
    @property
    def hall_number(self) -> int: ...
    @property
    def linear(self) -> list[list[int]]: ...
    @property
    def origin_shift(self) -> list[float]: ...
    # Serialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""

class MagneticSpaceGroup:
    def __init__(
        self,
        prim_rotations: list[list[int]],
        prim_translations: list[list[float]],
        prim_time_reversals: list[bool],
        *,
        basis: list[list[float]] | None = None,
        epsilon: float = 1e-4,
    ): ...
    @property
    def uni_number(self) -> int: ...
    @property
    def linear(self) -> list[list[int]]: ...
    @property
    def origin_shift(self) -> list[float]: ...
    # Serialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""



from __future__ import annotations

from typing import TYPE_CHECKING

import numpy as np
import pytest
from pymatgen.core import Composition, DummySpecies, Element, Species, Structure

from moyopy.interface import MoyoAdapter, MoyoNonCollinearMagneticAdapter

if TYPE_CHECKING:
    from moyopy import Cell, NonCollinearMagneticCell


def test_pymatgen_moyopy(wurtzite: Cell):
    structure = MoyoAdapter.get_structure(wurtzite)
    cell = MoyoAdapter.from_structure(structure)
    assert np.allclose(cell.basis, wurtzite.basis)
    assert np.allclose(cell.positions, wurtzite.positions)
    assert cell.numbers == wurtzite.numbers


def test_pymatgen_disordered_moyopy():
    structure = Structure(
        lattice=4.0 * np.eye(3),
        coords=[
            [0.0, 0.0, 0.0],
            [0.0, 0.5, 0.5],
            [0.5, 0.0, 0.5],
            [0.5, 0.5, 0.0],
        ],
        species=[
            Element("Cu"),
            Species("Cu2+"),
            DummySpecies("X0+"),
            Composition("Cu0.5Au0.5"),
        ],
    )
    cell, unique_species_mapping = MoyoAdapter.from_disordered_structure(structure)
    structure2 = MoyoAdapter.get_structure(cell, unique_species_mapping=unique_species_mapping)
    assert np.allclose(structure.lattice.matrix, structure2.lattice.matrix)
    assert np.allclose(structure.frac_coords, structure2.frac_coords)
    assert structure.species_and_occu == structure2.species_and_occu


def test_ase_moyopy(wurtzite: Cell):
    atoms = MoyoAdapter.get_atoms(wurtzite)
    cell = MoyoAdapter.from_atoms(atoms)
    assert np.allclose(cell.basis, wurtzite.basis)
    assert np.allclose(cell.positions, wurtzite.positions)
    assert cell.numbers == wurtzite.numbers


def test_from_py_obj(wurtzite: Cell):
    # Create different structure types
    structure = MoyoAdapter.get_structure(wurtzite)
    atoms = MoyoAdapter.get_atoms(wurtzite)
    mson_atoms = structure.to_ase_atoms()

    # Test conversion from each type
    for struct in [structure, atoms, mson_atoms]:
        cell = MoyoAdapter.from_py_obj(struct)
        assert np.allclose(cell.basis, wurtzite.basis)
        assert np.allclose(cell.positions, wurtzite.positions)
        assert cell.numbers == wurtzite.numbers

    # Test invalid input type
    with pytest.raises(TypeError, match="Expected Structure, Atoms, or MSONAtoms, got list"):
        MoyoAdapter.from_py_obj([1, 2, 3])


def test_noncollinear_adapter(rutile_type3_noncollinear: NonCollinearMagneticCell):
    structure = MoyoNonCollinearMagneticAdapter.get_structure(rutile_type3_noncollinear)
    magnetic_cell = MoyoNonCollinearMagneticAdapter.from_structure(structure)
    assert np.allclose(magnetic_cell.basis, rutile_type3_noncollinear.basis)
    assert np.allclose(magnetic_cell.positions, rutile_type3_noncollinear.positions)
    assert magnetic_cell.numbers == rutile_type3_noncollinear.numbers
    assert np.allclose(magnetic_cell.magnetic_moments, rutile_type3_noncollinear.magnetic_moments)



from moyopy import Cell


def test_cell_serialization(wurtzite: Cell):
    serialized = wurtzite.serialize_json()
    deserialized = Cell.deserialize_json(serialized)
    assert len(wurtzite.positions) == len(deserialized.positions)


def test_cell_py_obj_serialization(wurtzite: Cell):
    deserialized = wurtzite.as_dict()
    serialized = Cell.from_dict(deserialized)
    assert len(wurtzite.positions) == len(serialized.positions)



# ruff: noqa: E501
from moyopy import SpaceGroupType

map_hm_short_to_number = {
    # SpaceGroupType(number).hm_short is separated by a space like "F m -3 m"
    SpaceGroupType(number).hm_short.replace(" ", ""): number
    for number in range(1, 230 + 1)
}

print(map_hm_short_to_number.get("Fm-3m"))  # -> 225
print(map_hm_short_to_number.get("P6_3/m"))  # -> 176
print(map_hm_short_to_number.get("Fmmm"))  # -> 69



from moyopy import MagneticSpaceGroupType


def test_magnetic_space_group_type():
    msgt = MagneticSpaceGroupType(1262)
    assert msgt.bns_number == "151.32"
    assert msgt.og_number == "153.4.1270"
    assert msgt.construct_type == 4



from __future__ import annotations

from warnings import warn

try:
    import ase
    import numpy as np
    from pymatgen.core import Composition, Element, Structure
    from pymatgen.io.ase import MSONAtoms
except ImportError:
    raise ImportError("Try installing dependencies with `pip install moyopy[interface]`")

import moyopy


class MoyoAdapter:
    @staticmethod
    def get_structure(
        cell: moyopy.Cell, *, unique_species_mapping: dict[int, Composition] | None = None
    ) -> Structure:
        """Convert a Moyo Cell to a pymatgen Structure.

        If the Cell was created from a disordered Structure, the unique_species_mapping should be
        provided to reconstruct the original species.

        Parameters
        ----------
        cell : moyopy.Cell
            The Moyo Cell to convert.
        unique_species_mapping : dict[int, Composition] | None
            A mapping from integer indices used in the Moyo Cell to the original pymatgen
            SpeciesLike objects. If None, assumes the Cell was created from an ordered Structure.

        Returns
        -------
        structure : Structure
            The converted pymatgen Structure object.
        """
        species = _species_from_numbers(cell.numbers, unique_species_mapping)
        return Structure(lattice=cell.basis, species=species, coords=cell.positions)

    @staticmethod
    def get_atoms(cell: moyopy.Cell) -> ase.Atoms:
        atoms = ase.Atoms(
            cell=cell.basis,
            scaled_positions=cell.positions,
            numbers=cell.numbers,
            pbc=True,
        )
        return atoms

    @staticmethod
    def from_structure(structure: Structure) -> moyopy.Cell:
        """Convert a pymatgen Structure to a Moyo Cell."""
        if not structure.is_ordered:
            raise ValueError("Structure must be ordered. Use from_disordered_structure instead.")

        basis = structure.lattice.matrix.tolist()
        positions = structure.frac_coords.tolist()
        numbers = [site.specie.Z for site in structure]

        return moyopy.Cell(
            basis=basis,
            positions=positions,
            numbers=numbers,
        )

    @staticmethod
    def from_disordered_structure(
        structure: Structure,
    ) -> tuple[moyopy.Cell, dict[int, Composition]]:
        """Convert a disordered pymatgen Structure to a Moyo Cell.

        Parameters
        ----------
        structure : Structure
            A pymatgen Structure object, which may contain disordered sites.

        Returns
        -------
        cell : moyopy.Cell
            The converted Moyo Cell object.
        unique_species_mapping : dict[int, Composition]
            A mapping from integer indices used in the Moyo Cell to the original pymatgen
            SpeciesLike objects.
        """
        if structure.is_ordered:
            warn("Structure is ordered. Consider using from_structure.")

        unique_species, unique_species_mapping = _get_unique_species_and_mapping(structure)

        basis = structure.lattice.matrix.tolist()
        positions = structure.frac_coords.tolist()
        numbers = [unique_species[site.species] for site in structure]
        cell = moyopy.Cell(
            basis=basis,
            positions=positions,
            numbers=numbers,
        )

        return cell, unique_species_mapping

    @staticmethod
    def from_atoms(atoms: ase.Atoms) -> moyopy.Cell:
        basis = list(atoms.cell)
        positions = atoms.get_scaled_positions()
        numbers = atoms.get_atomic_numbers()

        return moyopy.Cell(
            basis=basis,
            positions=positions,
            numbers=numbers,
        )

    @staticmethod
    def from_py_obj(struct: Structure | ase.Atoms | MSONAtoms) -> moyopy.Cell:
        """Convert a Python atomic structure object to a Moyo Cell.

        Args:
            struct: Currently supports pymatgen Structure, ASE Atoms, and MSONAtoms

        Returns:
            moyopy.Cell: The converted Moyo cell
        """
        if isinstance(struct, (ase.Atoms, MSONAtoms)):
            return MoyoAdapter.from_atoms(struct)
        elif isinstance(struct, Structure):
            return MoyoAdapter.from_structure(struct)
        else:
            cls_name = type(struct).__name__
            raise TypeError(f"Expected Structure, Atoms, or MSONAtoms, got {cls_name}")


class MoyoNonCollinearMagneticAdapter:
    @staticmethod
    def get_structure(
        magnetic_cell: moyopy.NonCollinearMagneticCell,
        *,
        unique_species_mapping: dict[int, Composition] | None = None,
    ) -> Structure:
        """Convert a Moyo NonCollinearMagneticCell to a pymatgen Structure.

        If the NonCollinearMagneticCell was created from a disordered Structure, the
        unique_species_mapping should be provided to reconstruct the original species.

        Parameters
        ----------
        magnetic_cell : moyopy.NonCollinearMagneticCell
            The Moyo NonCollinearMagneticCell to convert.
        unique_species_mapping : dict[int, Composition] | None
            A mapping from integer indices used in the Moyo NonCollinearMagneticCell to the
            original pymatgen SpeciesLike objects. If None, assumes the NonCollinearMagneticCell
            was created from an ordered Structure.

        Returns
        -------
        structure : Structure
            The converted pymatgen Structure object with magnetic moments in
            site_properties['magmom'].
        """
        species = _species_from_numbers(magnetic_cell.numbers, unique_species_mapping)
        return Structure(
            lattice=magnetic_cell.basis,
            species=species,
            coords=magnetic_cell.positions,
            site_properties={"magmom": magnetic_cell.magnetic_moments},
        )

    @staticmethod
    def from_structure(structure: Structure) -> moyopy.NonCollinearMagneticCell:
        """Convert a pymatgen Structure with non-collinear magnetic moments to a Moyo
        NonCollinearMagneticCell."""
        if not structure.is_ordered:
            raise ValueError("Structure must be ordered. Use from_disordered_structure instead.")

        basis = structure.lattice.matrix.tolist()
        positions = structure.frac_coords.tolist()
        numbers = [site.specie.Z for site in structure]
        if "magmom" not in structure.site_properties:
            raise ValueError(
                "Structure must have non-collinear magnetic moments in site_properties['magmom']"
            )
        magnetic_moments_arr = np.array(structure.site_properties["magmom"])

        if magnetic_moments_arr.shape != (len(structure), 3):
            raise ValueError(
                f"Structure must have non-collinear magnetic moments. "
                f"Expected: ({len(structure)}, 3), actual: {magnetic_moments_arr.shape}."
            )

        return moyopy.NonCollinearMagneticCell(
            basis=basis,
            positions=positions,
            numbers=numbers,
            magnetic_moments=magnetic_moments_arr.tolist(),
        )

    @staticmethod
    def from_disordered_structure(
        structure: Structure,
    ) -> tuple[moyopy.NonCollinearMagneticCell, dict[int, Composition]]:
        """Convert a disordered pymatgen Structure with non-collinear magnetic moments to a Moyo
        NonCollinearMagneticCell.

        Parameters
        ----------
        structure : Structure
            A pymatgen Structure object, which may contain disordered sites. Must have
            non-collinear magnetic moments in site_properties['magmom'].

        Returns
        -------
        magnetic_cell : moyopy.NonCollinearMagneticCell
            The converted Moyo NonCollinearMagneticCell object.
        unique_species_mapping : dict[int, Composition]
            A mapping from integer indices used in the Moyo NonCollinearMagneticCell to the
            original pymatgen SpeciesLike objects.
        """
        if structure.is_ordered:
            warn("Structure is ordered. Consider using from_structure.")

        unique_species, unique_species_mapping = _get_unique_species_and_mapping(structure)

        basis = structure.lattice.matrix.tolist()
        positions = structure.frac_coords.tolist()
        numbers = [unique_species[site.species] for site in structure]
        if "magmom" not in structure.site_properties:
            raise ValueError(
                "Structure must have non-collinear magnetic moments in site_properties['magmom']"
            )
        magnetic_moments_arr = np.array(structure.site_properties["magmom"])

        if magnetic_moments_arr.shape != (len(structure), 3):
            raise ValueError(
                "Structure must have non-collinear magnetic moments in site_properties['magmom']. "
                f"Expected shape: ({len(structure)}, 3), got: {magnetic_moments_arr.shape}."
            )

        magnetic_cell = moyopy.NonCollinearMagneticCell(
            basis=basis,
            positions=positions,
            numbers=numbers,
            magnetic_moments=magnetic_moments_arr.tolist(),
        )
        return magnetic_cell, unique_species_mapping


def _species_from_numbers(
    numbers: list[int], unique_species_mapping: dict[int, Composition] | None = None
) -> list[Composition]:
    if unique_species_mapping:
        species = [unique_species_mapping[number] for number in numbers]
    else:
        species = [Element.from_Z(number) for number in numbers]
    return species


def _get_unique_species_and_mapping(
    structure: Structure,
) -> tuple[dict[Composition, int], dict[int, Composition]]:
    unique_species = {}
    for site in structure:
        key = site.species
        if key not in unique_species:
            unique_species[key] = len(unique_species)
    unique_species_mapping = {i: species for i, species in enumerate(unique_species)}
    return unique_species, unique_species_mapping



from typing import Any

class Cell:
    def __init__(
        self,
        basis: list[list[float]],
        positions: list[list[float]],
        numbers: list[int],
    ): ...
    @property
    def basis(self) -> list[list[float]]: ...
    @property
    def positions(self) -> list[list[float]]: ...
    @property
    def numbers(self) -> list[int]: ...
    @property
    def num_atoms(self) -> int: ...
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    @classmethod
    def deserialize_json(cls, json_str: str) -> Cell:
        """Deserialize a JSON string to an object"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""
    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> Cell:
        """Create an object from a dictionary"""

class CollinearMagneticCell:
    def __init__(
        self,
        basis: list[list[float]],
        positions: list[list[float]],
        numbers: list[int],
        magnetic_moments: list[float],
    ): ...
    @property
    def basis(self) -> list[list[float]]: ...
    @property
    def positions(self) -> list[list[float]]: ...
    @property
    def numbers(self) -> list[int]: ...
    @property
    def magnetic_moments(self) -> list[float]: ...
    @property
    def num_atoms(self) -> int: ...
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    @classmethod
    def deserialize_json(cls, json_str: str) -> CollinearMagneticCell:
        """Deserialize a JSON string to an object"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""
    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> CollinearMagneticCell:
        """Create an object from a dictionary"""

class NonCollinearMagneticCell:
    def __init__(
        self,
        basis: list[list[float]],
        positions: list[list[float]],
        numbers: list[int],
        magnetic_moments: list[list[float]],
    ): ...
    @property
    def basis(self) -> list[list[float]]: ...
    @property
    def positions(self) -> list[list[float]]: ...
    @property
    def numbers(self) -> list[int]: ...
    @property
    def magnetic_moments(self) -> list[list[float]]: ...
    @property
    def num_atoms(self) -> int: ...
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    @classmethod
    def deserialize_json(cls, json_str: str) -> NonCollinearMagneticCell:
        """Deserialize a JSON string to an object"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""
    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> NonCollinearMagneticCell:
        """Create an object from a dictionary"""

class Operations:
    @property
    def rotations(self) -> list[list[list[float]]]: ...
    @property
    def translations(self) -> list[list[float]]: ...
    @property
    def num_operations(self) -> int: ...
    def __len__(self) -> int: ...
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    @classmethod
    def deserialize_json(cls, json_str: str) -> Operations:
        """Deserialize a JSON string to an object"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""
    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> Operations:
        """Create an object from a dictionary"""

class MagneticOperations:
    @property
    def rotations(self) -> list[list[list[float]]]: ...
    @property
    def translations(self) -> list[list[float]]: ...
    @property
    def time_reversals(self) -> list[bool]: ...
    @property
    def num_operations(self) -> int: ...
    def __len__(self) -> int: ...
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string"""
    @classmethod
    def deserialize_json(cls, json_str: str) -> MagneticOperations:
        """Deserialize a JSON string to an object"""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary"""
    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> MagneticOperations:
        """Create an object from a dictionary"""



from __future__ import annotations

import pytest

from moyopy import (
    MagneticSpaceGroup,
    PointGroup,
    SpaceGroup,
    magnetic_operations_from_uni_number,
    operations_from_number,
)


@pytest.mark.parametrize(
    "number,arithmetic_number",
    [
        (158, 45),  # P3c1, 3m1P
        (159, 46),  # P31c, 31mP
    ],
)
def test_identify_point_group(number: int, arithmetic_number: int):
    operations = operations_from_number(number, primitive=True)
    point_group = PointGroup(operations.rotations)
    assert point_group.arithmetic_number == arithmetic_number


@pytest.mark.parametrize(
    "number",
    [
        158,  # P3c1
        159,  # P31c
    ],
)
def test_identify_space_group(number: int):
    operations = operations_from_number(number, primitive=True)
    space_group = SpaceGroup(
        prim_rotations=operations.rotations, prim_translations=operations.translations
    )
    assert space_group.number == number


@pytest.mark.parametrize(
    "uni_number",
    [
        1242,  # R31'_c
    ],
)
def test_identify_magnetic_space_group(uni_number: int):
    magnetic_operations = magnetic_operations_from_uni_number(uni_number, primitive=True)
    magnetic_space_group = MagneticSpaceGroup(
        prim_rotations=magnetic_operations.rotations,
        prim_translations=magnetic_operations.translations,
        prim_time_reversals=magnetic_operations.time_reversals,
    )
    assert magnetic_space_group.uni_number == uni_number



from __future__ import annotations

from moyopy import Cell, CollinearMagneticCell, MoyoCollinearMagneticDataset, MoyoDataset


def test_moyo_dataset(wurtzite: Cell):
    dataset = MoyoDataset(wurtzite)
    assert dataset.number == 186
    assert dataset.hall_number == 480
    assert dataset.pearson_symbol == "hP4"


def test_moyo_dataset_serialization(wurtzite: Cell):
    dataset = MoyoDataset(wurtzite)
    serialized = dataset.serialize_json()
    deserialized = MoyoDataset.deserialize_json(serialized)
    assert deserialized.number == dataset.number
    assert deserialized.std_cell.num_atoms == dataset.std_cell.num_atoms


def test_moyo_dataset_py_obj_serialization(wurtzite: Cell):
    dataset = MoyoDataset(wurtzite)
    deserialized = dataset.as_dict()
    serialized = MoyoDataset.from_dict(deserialized)
    assert serialized.number == dataset.number
    assert serialized.std_cell.num_atoms == dataset.std_cell.num_atoms


def test_moyo_dataset_repr(wurtzite: Cell):
    dataset = MoyoDataset(wurtzite)
    dataset_str = str(dataset)

    # Test that string representation of MoyoDataset contains key information
    assert "MoyoDataset" in dataset_str
    assert f"number={dataset.number}" in dataset_str
    assert f"hall_number={dataset.hall_number}" in dataset_str
    assert f"operations=<{len(dataset.operations)} operations>" in dataset_str
    assert f"orbits={dataset.orbits}" in dataset_str
    assert f"wyckoffs={dataset.wyckoffs}" in dataset_str

    # Test site_symmetry_symbols content without caring about quote style
    symbols = dataset.site_symmetry_symbols
    assert all(symbol in dataset_str for symbol in symbols)
    assert str(len(symbols)) in dataset_str

    # Test that repr() gives different output
    assert str(dataset) != repr(dataset)


def test_moyo_collinear_magnetic_dataset(rutile_type3: CollinearMagneticCell):
    uni_number = 1158
    dataset = MoyoCollinearMagneticDataset(rutile_type3)
    assert dataset.uni_number == uni_number
    dataset2 = MoyoCollinearMagneticDataset(dataset.std_mag_cell)
    assert dataset2.uni_number == uni_number
    dataset3 = MoyoCollinearMagneticDataset(dataset.prim_std_mag_cell)
    assert dataset3.uni_number == uni_number



from typing import Any

from typing_extensions import Self

from moyopy._base import (
    Cell,
    CollinearMagneticCell,
    MagneticOperations,
    NonCollinearMagneticCell,
    Operations,
)
from moyopy._data import (
    Setting,
)

class MoyoDataset:
    """A dataset containing symmetry information of the input crystal structure."""
    def __init__(
        self,
        cell: Cell,
        *,
        symprec: float = 1e-4,
        angle_tolerance: float | None = None,
        setting: Setting | None = None,
        rotate_basis: bool = True,
    ):
        """
        Parameters
        ----------
        cell: Cell
            Input crystal structure.
        symprec: float
            Symmetry search tolerance in the unit of cell.basis.
        angle_tolerance: float | None
            Symmetry search tolerance in the unit of radians.
        setting: Setting | None
            Preference for the setting of the space group.
        rotate_basis: bool
            Whether to rotate the basis vectors of the input cell to those of the standardized
            cell.
        """
    # Space-group type
    @property
    def number(self) -> int:
        """Space group number."""
    @property
    def hall_number(self) -> int:
        """Hall symbol number."""
    # Symmetry operations in the input cell
    @property
    def operations(self) -> Operations:
        """Symmetry operations in the input cell."""
    # Site symmetry
    @property
    def orbits(self) -> list[int]:
        """Spglib's `crystallographic_orbits` not `equivalent_atoms`.

        The `i`th atom in the input cell is equivalent to the `orbits[i]`th atom in the **input**
        cell. For example, orbits=[0, 0, 2, 2, 2, 2] means the first two atoms are equivalent
        and the last four atoms are equivalent to each other.
        """
    @property
    def wyckoffs(self) -> list[str]:
        """Wyckoff letters for each site in the input cell."""
    @property
    def site_symmetry_symbols(self) -> list[str]:
        """Site symmetry symbols for each site in the input cell.

        The orientation of the site symmetry is w.r.t. the standardized cell.
        """
    # Standardized cell
    @property
    def std_cell(self) -> Cell:
        """Standardized cell."""
    @property
    def std_linear(self) -> list[list[float]]:
        """Linear part of transformation from the input cell to the standardized cell."""
    @property
    def std_origin_shift(self) -> list[float]:
        """Origin shift of transformation from the input cell to the standardized cell."""
    @property
    def std_rotation_matrix(self) -> list[list[float]]:
        """Rigid rotation."""
    @property
    def pearson_symbol(self) -> str:
        """Pearson symbol for standardized cell."""
    # Primitive standardized cell
    @property
    def prim_std_cell(self) -> Cell:
        """Primitive standardized cell."""
    @property
    def prim_std_linear(self) -> list[list[float]]:
        """Linear part of transformation from the input cell to the primitive standardized cell."""
    @property
    def prim_std_origin_shift(self) -> list[float]:
        """Origin shift of transformation from the input cell to the primitive standardized
        cell."""
    @property
    def mapping_std_prim(self) -> list[int]:
        """Mapping sites in the input cell to those in the primitive standardized cell.

        The `i`th atom in the input cell is mapped to the `mapping_to_std_prim[i]`th atom in the
        primitive standardized cell.
        """
    # Final parameters
    @property
    def symprec(self) -> float:
        """Actually used `symprec` in iterative symmetry search."""
    @property
    def angle_tolerance(self) -> float | None:
        """Actually used `angle_tolerance` in iterative symmetry search."""
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string."""
    @classmethod
    def deserialize_json(cls, json_str: str) -> Self:
        """Deserialize an object from a JSON string."""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary."""
    @classmethod
    def from_dict(cls, obj: dict[str, Any]) -> Self:
        """Create an object from a dictionary."""

class MoyoCollinearMagneticDataset:
    """A dataset containing magnetic symmetry information of the input collinear magnetic
    structure."""
    def __init__(
        self,
        magnetic_cell: CollinearMagneticCell,
        *,
        symprec: float = 1e-4,
        angle_tolerance: float | None = None,
        mag_symprec: float | None = None,
        is_axial: bool = False,
        rotate_basis: bool = True,
    ):
        """
        Parameters
        ----------
        magnetic_cell: CollinearMagneticCell
            Input collinear magnetic structure.
        symprec: float
            Symmetry search tolerance in the unit of magnetic_cell.basis.
        angle_tolerance: float | None
            Symmetry search tolerance in the unit of radians.
        mag_symprec: float | None
            Symmetry search tolerance in the unit of magnetic moments.
        is_axial: bool
            Whether the magnetic moments are axial on improper operations.
        rotate_basis: bool
            Whether to rotate the basis vectors of the input cell to those of the standardized
            cell.
        """
    # Magnetic space-group type
    @property
    def uni_number(self) -> int:
        """UNI number for magnetic space-group type."""
    # Magnetic symmetry operations in the input cell
    @property
    def magnetic_operations(self) -> MagneticOperations:
        """Magnetic symmetry operations in the input cell."""
    # Site symmetry
    @property
    def orbits(self) -> list[int]:
        """The `i`th atom in the input magnetic cell is equivalent to the `orbits[i]`th atom
        in the **input** magnetic cell. For example, orbits=[0, 0, 2, 2, 2, 2] means
        the first two atoms are equivalent and the last four atoms are equivalent to each other.
        """
    # Standardized magnetic cell
    @property
    def std_mag_cell(self) -> CollinearMagneticCell:
        """Standardized magnetic cell."""
    @property
    def std_linear(self) -> list[list[float]]:
        """Linear part of transformation from the input magnetic cell to the standardized
        magnetic cell."""
    @property
    def std_origin_shift(self) -> list[float]:
        """Origin shift of transformation from the input magnetic cell to the standardized
        magnetic cell."""
    @property
    def std_rotation_matrix(self) -> list[list[float]]:
        """Rigid rotation."""
    # Primitive standardized magnetic cell
    @property
    def prim_std_mag_cell(self) -> CollinearMagneticCell:
        """Primitive standardized magnetic cell."""
    @property
    def prim_std_linear(self) -> list[list[float]]:
        """Linear part of transformation from the input magnetic cell to the primitive
        standardized magnetic cell."""
    @property
    def prim_std_origin_shift(self) -> list[float]:
        """Origin shift of transformation from the input magnetic cell to the primitive
        standardized magnetic cell."""
    @property
    def mapping_std_prim(self) -> list[int]:
        """Mapping sites in the input magnetic cell to those in the primitive standardized magnetic
        cell. The `i`th atom in the input magnetic cell is mapped to the `mapping_to_std_prim[i]`th
        atom in the primitive standardized magnetic cell.
        """
    # Final parameters
    @property
    def symprec(self) -> float:
        """Actually used `symprec` in iterative symmetry search."""
    @property
    def angle_tolerance(self) -> float | None:
        """Actually used `angle_tolerance` in iterative symmetry search."""
    @property
    def mag_symprec(self) -> float | None:
        """Actually used `mag_symprec` in iterative symmetry search."""
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string."""
    @classmethod
    def deserialize_json(cls, json_str: str) -> Self:
        """Deserialize an object from a JSON string."""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary."""
    @classmethod
    def from_dict(cls, obj: dict[str, Any]) -> Self:
        """Create an object from a dictionary."""

class MoyoNonCollinearMagneticDataset:
    """A dataset containing magnetic symmetry information of the input non-collinear magnetic
    structure."""
    def __init__(
        self,
        magnetic_cell: NonCollinearMagneticCell,
        *,
        symprec: float = 1e-4,
        angle_tolerance: float | None = None,
        mag_symprec: float | None = None,
        is_axial: bool = True,
        rotate_basis: bool = True,
    ):
        """
        Parameters
        ----------
        magnetic_cell: NonCollinearMagneticCell
            Input non-collinear magnetic structure.
        symprec: float
            Symmetry search tolerance in the unit of magnetic_cell.basis.
        angle_tolerance: float | None
            Symmetry search tolerance in the unit of radians.
        mag_symprec: float | None
            Symmetry search tolerance in the unit of magnetic moments.
        is_axial: bool
            Whether the magnetic moments are axial on improper operations.
        rotate_basis: bool
            Whether to rotate the basis vectors of the input cell to those of the standardized
            cell.
        """
    # Magnetic space-group type
    @property
    def uni_number(self) -> int:
        """UNI number for magnetic space-group type."""
    # Magnetic symmetry operations in the input cell
    @property
    def magnetic_operations(self) -> MagneticOperations:
        """Magnetic symmetry operations in the input cell."""
    # Site symmetry
    @property
    def orbits(self) -> list[int]:
        """The `i`th atom in the input magnetic cell is equivalent to the `orbits[i]`th atom
        in the **input** magnetic cell. For example, orbits=[0, 0, 2, 2, 2, 2] means
        the first two atoms are equivalent and the last four atoms are equivalent to each other.
        """
    # Standardized magnetic cell
    @property
    def std_mag_cell(self) -> NonCollinearMagneticCell:
        """Standardized magnetic cell."""
    @property
    def std_linear(self) -> list[list[float]]:
        """Linear part of transformation from the input magnetic cell to the standardized
        magnetic cell."""
    @property
    def std_origin_shift(self) -> list[float]:
        """Origin shift of transformation from the input magnetic cell to the standardized
        magnetic cell."""
    @property
    def std_rotation_matrix(self) -> list[list[float]]:
        """Rigid rotation."""
    # Primitive standardized magnetic cell
    @property
    def prim_std_mag_cell(self) -> NonCollinearMagneticCell:
        """Primitive standardized magnetic cell."""
    @property
    def prim_std_linear(self) -> list[list[float]]:
        """Linear part of transformation from the input magnetic cell to the primitive
        standardized magnetic cell."""
    @property
    def prim_std_origin_shift(self) -> list[float]:
        """Origin shift of transformation from the input magnetic cell to the primitive
        standardized magnetic cell."""
    @property
    def mapping_std_prim(self) -> list[int]:
        """Mapping sites in the input magnetic cell to those in the primitive standardized magnetic
        cell. The `i`th atom in the input magnetic cell is mapped to the `mapping_to_std_prim[i]`th
        atom in the primitive standardized magnetic cell.
        """
    # Final parameters
    @property
    def symprec(self) -> float:
        """Actually used `symprec` in iterative symmetry search."""
    @property
    def angle_tolerance(self) -> float | None:
        """Actually used `angle_tolerance` in iterative symmetry search."""
    @property
    def mag_symprec(self) -> float | None:
        """Actually used `mag_symprec` in iterative symmetry search."""
    # Serialization and deserialization
    def serialize_json(self) -> str:
        """Serialize an object to a JSON string."""
    @classmethod
    def deserialize_json(cls, json_str: str) -> Self:
        """Deserialize an object from a JSON string."""
    def as_dict(self) -> dict[str, Any]:
        """Convert an object to a dictionary."""
    @classmethod
    def from_dict(cls, obj: dict[str, Any]) -> Self:
        """Create an object from a dictionary."""



from __future__ import annotations

from moyopy import ArithmeticCrystalClass


def test_arithmetic_crystal_class():
    a1 = ArithmeticCrystalClass(45)  # 3m1P
    assert a1.arithmetic_number == 45
    assert a1.symbol == "3m1P"
    assert a1.geometric_crystal_class == "3m"
    assert a1.bravais_class == "hP"
